@@ -111,6 +111,10 @@ class StreamResponse(HeadersMixin):
     def enable_chunked_encoding(self, chunk_size=None):
         """Enables automatic chunked transfer encoding."""
         self._chunked = True
+
+        if hdrs.CONTENT_LENGTH in self._headers:
+            raise RuntimeError("You can't enable chunked encoding when "
+                               "a content length is set")
         if chunk_size is not None:
             warnings.warn('Chunk size is deprecated #1615', DeprecationWarning)
 
@@ -194,7 +198,9 @@ class StreamResponse(HeadersMixin):
     def content_length(self, value):
         if value is not None:
             value = int(value)
-            # TODO: raise error if chunked enabled
+            if self._chunked:
+                raise RuntimeError("You can't set content length when "
+                                   "chunked encoding is enable")
             self._headers[hdrs.CONTENT_LENGTH] = str(value)
         else:
             self._headers.pop(hdrs.CONTENT_LENGTH, None)
@@ -563,6 +569,8 @@ class Response(StreamResponse):
 
     @asyncio.coroutine
     def write_eof(self):
+        if self._eof_sent:
+            return
         body = self._body
         if body is not None:
             if (self._req._method == hdrs.METH_HEAD or
